@@ -1,4 +1,5 @@
-{-# LANGUAGE CPP, BangPatterns, ScopedTypeVariables, ForeignFunctionInterface #-}
+{-# LANGUAGE BangPatterns, ScopedTypeVariables #-}
+{-# OPTIONS_HADDOCK hide #-}
 -- |
 -- Module    : Numeric.SpecFunctions.Internal
 -- Copyright : (c) 2009, 2011, 2012 Bryan O'Sullivan
@@ -9,11 +10,13 @@
 -- Portability : portable
 --
 -- Internal module with implementation of special functions.
-module Numeric.SpecFunctions.Internal where
+module Numeric.SpecFunctions.Internal
+    ( module Numeric.SpecFunctions.Internal
+    , Compat.log1p
+    , Compat.expm1
+    ) where
 
-#if !MIN_VERSION_base(4,9,0)
 import Control.Applicative
-#endif
 import Data.Bits          ((.&.), (.|.), shiftR)
 import Data.Int           (Int64)
 import Data.Word          (Word)
@@ -21,17 +24,15 @@ import Data.Default.Class
 import qualified Data.Vector.Unboxed as U
 import           Data.Vector.Unboxed   ((!))
 import Text.Printf
-#if MIN_VERSION_base(4,9,0)
-import GHC.Float (log1p,expm1)
-#endif
 
 import Numeric.Polynomial.Chebyshev    (chebyshevBroucke)
-import Numeric.Polynomial              (evaluatePolynomialL,evaluateEvenPolynomialL,evaluateOddPolynomialL)
+import Numeric.Polynomial              (evaluatePolynomial, evaluatePolynomialL, evaluateEvenPolynomialL
+                                       ,evaluateOddPolynomialL)
 import Numeric.RootFinding             (Root(..), newtonRaphson, NewtonParam(..), Tolerance(..))
 import Numeric.Series
 import Numeric.MathFunctions.Constants
-
-
+import Numeric.SpecFunctions.Compat (log1p)
+import qualified Numeric.SpecFunctions.Compat as Compat
 
 ----------------------------------------------------------------
 -- Error function
@@ -53,8 +54,8 @@ import Numeric.MathFunctions.Constants
 -- \end{aligned}
 -- \]
 erf :: Double -> Double
+erf = Compat.erf
 {-# INLINE erf #-}
-erf = c_erf
 
 -- | Complementary error function.
 --
@@ -72,17 +73,26 @@ erf = c_erf
 -- \end{aligned}
 -- \]
 erfc :: Double -> Double
+erfc = Compat.erfc
 {-# INLINE erfc #-}
-erfc = c_erfc
-
-foreign import ccall "erf"  c_erf  :: Double -> Double
-foreign import ccall "erfc" c_erfc :: Double -> Double
-
 
 -- | Inverse of 'erf'.
 invErf :: Double -- ^ /p/ ∈ [-1,1]
        -> Double
-invErf p = invErfc (1 - p)
+invErf p
+  | p ==  1         = m_pos_inf
+  | p == -1         = m_neg_inf
+  | p < 1 && p > -1 = if p > 0 then r else -r
+  | otherwise       = error "invErf: p must in [-1,1] range"
+  where
+    -- We solve equation same as in invErfc. We're able to ruse same
+    -- Halley step by solving equation:
+    --   > pp - erf x = 0
+    -- instead of
+    --   > erf x - pp = 0
+    pp     = abs p
+    r      = step $ step $ guessInvErfc $ 1 - pp
+    step x = invErfcHalleyStep (pp - erf x) x
 
 -- | Inverse of 'erfc'.
 invErfc :: Double -- ^ /p/ ∈ [0,2]
@@ -93,20 +103,23 @@ invErfc p
   | p >0 && p < 2 = if p <= 1 then r else -r
   | otherwise     = modErr $ "invErfc: p must be in [0,2] got " ++ show p
   where
-    pp = if p <= 1 then p else 2 - p
-    t  = sqrt $ -2 * log( 0.5 * pp)
-    -- Initial guess
-    x0 = -0.70711 * ((2.30753 + t * 0.27061) / (1 + t * (0.99229 + t * 0.04481)) - t)
-    r  = loop 0 x0
-    --
-    loop :: Int -> Double -> Double
-    loop !j !x
-      | j >= 2    = x
-      | otherwise = let err = erfc x - pp
-                        x'  = x + err / (1.12837916709551257 * exp(-x * x) - x * err) -- // Halley
-                    in loop (j+1) x'
+    pp | p <= 1    = p
+       | otherwise = 2 - p
+    -- We perform 2 Halley steps in order to get to solution
+    r      = step $ step $ guessInvErfc pp
+    step x = invErfcHalleyStep (erfc x - pp) x
 
+-- Initial guess for invErfc & invErf
+guessInvErfc :: Double -> Double
+guessInvErfc p
+  = -0.70711 * ((2.30753 + t * 0.27061) / (1 + t * (0.99229 + t * 0.04481)) - t)
+  where
+    t = sqrt $ -2 * log( 0.5 * p)
 
+-- Halley step for solving invErfc
+invErfcHalleyStep :: Double -> Double -> Double
+invErfcHalleyStep err x
+  = x + err / (1.12837916709551257 * exp(-x * x) - x * err)
 
 ----------------------------------------------------------------
 -- Gamma function
@@ -127,33 +140,205 @@ data L = L {-# UNPACK #-} !Double {-# UNPACK #-} !Double
 -- Returns &#8734; if the input is outside of the range (0 < /x/
 -- &#8804; 1e305).
 logGamma :: Double -> Double
-logGamma x
-  | x <= 0    = m_pos_inf
-  | x <  1    = lanczos (1+x) - log x
-  | otherwise = lanczos x
-  where
-    -- Evaluate Lanczos approximation for γ=6
-    lanczos z = fini
-              $ U.foldl' go (L 0 (z+7)) a
-      where
-        fini (L l _)   = log (l+a0) + log m_sqrt_2_pi - z65 + (z-0.5) * log z65
-        go   (L l t) k = L (l + k / t) (t-1)
-        z65 = z + 6.5
-    -- Coefficients for Lanczos approximation
-    a0  = 0.9999999999995183
-    a   = U.fromList [ 0.1659470187408462e-06
-                     , 0.9934937113930748e-05
-                     , -0.1385710331296526
-                     , 12.50734324009056
-                     , -176.6150291498386
-                     , 771.3234287757674
-                     , -1259.139216722289
-                     , 676.5203681218835
-                     ]
+logGamma z
+  | z <= 0    = m_pos_inf
+  -- For very small values z we can just use Laurent expansion
+  | z < m_sqrt_eps = log (1/z - m_eulerMascheroni)
+  -- For z<1 we use recurrence. Γ(z+1) = z·Γ(z) Note that in order to
+  -- avoid precision loss we have to compute parameter to
+  -- approximations here:
+  --
+  -- > (z + 1) - 1 = z
+  -- > (z + 1) - 2 = z - 1
+  --
+  -- Simple passing (z + 1) to piecewise approxiations and computing
+  -- difference leads to bad loss of precision near 1.
+  -- This is reason lgamma1_15 & lgamma15_2 have three parameters
+  | z < 0.5   = lgamma1_15 z (z - 1) - log z
+  | z < 1     = lgamma15_2 z (z - 1) - log z
+  -- Piecewise polynomial approximations
+  | z <= 1.5  = lgamma1_15 (z - 1) (z - 2)
+  | z < 2     = lgamma15_2 (z - 1) (z - 2)
+  | z < 15    = lgammaSmall z
+  -- Otherwise we switch to Lanczos approximation
+  | otherwise = lanczosApprox z
+
 
 -- | Synonym for 'logGamma'. Retained for compatibility
 logGammaL :: Double -> Double
 logGammaL = logGamma
+{-# DEPRECATED logGammaL "Use logGamma instead" #-}
+
+
+
+-- Polynomial expansion used in interval (1,1.5]
+--
+-- > logΓ(z) = (z-1)(z-2)(Y + R(z-1))
+lgamma1_15 :: Double -> Double -> Double
+lgamma1_15 zm1 zm2
+   = r * y + r * ( evaluatePolynomial zm1 tableLogGamma_1_15P
+                 / evaluatePolynomial zm1 tableLogGamma_1_15Q
+                 )
+   where
+     r = zm1 * zm2
+     y = 0.52815341949462890625
+
+tableLogGamma_1_15P,tableLogGamma_1_15Q :: U.Vector Double
+tableLogGamma_1_15P = U.fromList
+  [  0.490622454069039543534e-1
+  , -0.969117530159521214579e-1
+  , -0.414983358359495381969e0
+  , -0.406567124211938417342e0
+  , -0.158413586390692192217e0
+  , -0.240149820648571559892e-1
+  , -0.100346687696279557415e-2
+  ]
+{-# NOINLINE tableLogGamma_1_15P #-}
+tableLogGamma_1_15Q = U.fromList
+  [ 1
+  , 0.302349829846463038743e1
+  , 0.348739585360723852576e1
+  , 0.191415588274426679201e1
+  , 0.507137738614363510846e0
+  , 0.577039722690451849648e-1
+  , 0.195768102601107189171e-2
+  ]
+{-# NOINLINE tableLogGamma_1_15Q #-}
+
+
+
+-- Polynomial expansion used in interval (1.5,2)
+--
+-- > logΓ(z) = (2-z)(1-z)(Y + R(2-z))
+lgamma15_2 :: Double -> Double -> Double
+lgamma15_2 zm1 zm2
+   = r * y + r * ( evaluatePolynomial (-zm2) tableLogGamma_15_2P
+                 / evaluatePolynomial (-zm2) tableLogGamma_15_2Q
+                 )
+   where
+     r = zm1 * zm2
+     y = 0.452017307281494140625
+
+tableLogGamma_15_2P,tableLogGamma_15_2Q :: U.Vector Double
+tableLogGamma_15_2P = U.fromList
+  [ -0.292329721830270012337e-1
+  ,  0.144216267757192309184e0
+  , -0.142440390738631274135e0
+  ,  0.542809694055053558157e-1
+  , -0.850535976868336437746e-2
+  ,  0.431171342679297331241e-3
+  ]
+{-# NOINLINE tableLogGamma_15_2P #-}
+tableLogGamma_15_2Q = U.fromList
+  [  1
+  , -0.150169356054485044494e1
+  ,  0.846973248876495016101e0
+  , -0.220095151814995745555e0
+  ,  0.25582797155975869989e-1
+  , -0.100666795539143372762e-2
+  , -0.827193521891290553639e-6
+  ]
+{-# NOINLINE tableLogGamma_15_2Q #-}
+
+
+
+-- Polynomial expansion used in interval (2,3)
+--
+-- > logΓ(z) = (z - 2)(z + 1)(Y + R(z-2))
+lgamma2_3 :: Double -> Double
+lgamma2_3 z
+  = r * y + r * ( evaluatePolynomial zm2 tableLogGamma_2_3P
+                / evaluatePolynomial zm2 tableLogGamma_2_3Q
+                )
+  where
+    r   = zm2 * (z + 1)
+    zm2 = z - 2
+    y   = 0.158963680267333984375e0
+
+
+tableLogGamma_2_3P,tableLogGamma_2_3Q :: U.Vector Double
+tableLogGamma_2_3P = U.fromList
+  [ -0.180355685678449379109e-1
+  ,  0.25126649619989678683e-1
+  ,  0.494103151567532234274e-1
+  ,  0.172491608709613993966e-1
+  , -0.259453563205438108893e-3
+  , -0.541009869215204396339e-3
+  , -0.324588649825948492091e-4
+  ]
+{-# NOINLINE tableLogGamma_2_3P #-}
+tableLogGamma_2_3Q = U.fromList
+  [  1
+  ,  0.196202987197795200688e1
+  ,  0.148019669424231326694e1
+  ,  0.541391432071720958364e0
+  ,  0.988504251128010129477e-1
+  ,  0.82130967464889339326e-2
+  ,  0.224936291922115757597e-3
+  , -0.223352763208617092964e-6
+  ]
+{-# NOINLINE tableLogGamma_2_3Q #-}
+
+
+
+-- For small z we can just use Gamma function recurrence and reduce
+-- problem to interval [2,3] and use polynomial approximation
+-- there. Surpringly it gives very good precision
+lgammaSmall :: Double -> Double
+lgammaSmall = go 0
+  where
+    go acc z | z < 3     = acc + lgamma2_3 z
+             | otherwise = go (acc + log zm1) zm1
+             where
+               zm1 = z - 1
+
+
+-- Lanczos approximation for gamma function.
+--
+-- > Γ(z) = sqrt(2π)(z + g - 0.5)^(z - 0.5)·exp{-(z + g - 0.5)}·A_g(z)
+--
+-- Coeffients are taken from boost. Constants are absorbed into
+-- polynomial's coefficients.
+lanczosApprox :: Double -> Double
+lanczosApprox z
+  = (log (z + g - 0.5) - 1) * (z - 0.5)
+  + log (evalRatio tableLanczos z)
+  where
+    g = 6.024680040776729583740234375
+
+tableLanczos :: U.Vector (Double,Double)
+{-# NOINLINE tableLanczos #-}
+tableLanczos = U.fromList
+  [ (56906521.91347156388090791033559122686859    , 0)
+  , (103794043.1163445451906271053616070238554    , 39916800)
+  , (86363131.28813859145546927288977868422342    , 120543840)
+  , (43338889.32467613834773723740590533316085    , 150917976)
+  , (14605578.08768506808414169982791359218571    , 105258076)
+  , (3481712.15498064590882071018964774556468     , 45995730)
+  , (601859.6171681098786670226533699352302507    , 13339535)
+  , (75999.29304014542649875303443598909137092    , 2637558)
+  , (6955.999602515376140356310115515198987526    , 357423)
+  , (449.9445569063168119446858607650988409623    , 32670)
+  , (19.51992788247617482847860966235652136208    , 1925)
+  , (0.5098416655656676188125178644804694509993   , 66)
+  , (0.006061842346248906525783753964555936883222 , 1)
+  ]
+
+-- Evaluate rational function. Polynomials in both numerator and
+-- denominator must have same order. Function seems to be too specific
+-- so it's not exposed
+--
+-- Special care taken in order to avoid overflow for large values of x
+evalRatio :: U.Vector (Double,Double) -> Double -> Double
+evalRatio coef x
+  | x > 1     = fini $ U.foldl' stepL (L 0 0) coef
+  | otherwise = fini $ U.foldr' stepR (L 0 0) coef
+  where
+    fini (L num den) = num / den
+    stepR (a,b) (L num den) = L (num * x  + a) (den * x  + b)
+    stepL (L num den) (a,b) = L (num * rx + a) (den * rx + b)
+    rx = recip x
+
 
 
 -- |
@@ -208,7 +393,7 @@ incompleteGamma a x
   --
   -- See http://functions.wolfram.com/GammaBetaErf/GammaRegularized/06/01/05/01/01/
   | x < sqrt m_epsilon && a > 1
-    = x**a / a / exp (logGammaL a) * (1 - a*x / (a + 1))
+    = x**a / a / exp (logGamma a) * (1 - a*x / (a + 1))
   | x < 0.5 = case () of
     _| (-0.4)/log x < a  -> taylorSeriesP
      | otherwise         -> taylorSeriesComplQ
@@ -233,7 +418,7 @@ incompleteGamma a x
     -- Series for 1-Q(a,x). See [Temme1994] Eq. 5.5
     taylorSeriesComplQ
       = sumPowerSeries (-x) (scanSequence (/) 1 (enumSequenceFrom 1) / enumSequenceFrom a)
-      * x**a / exp(logGammaL a)
+      * x**a / exp(logGamma a)
     -- Legendre continued fractions
     contFraction = 1 - ( exp ( log x * a - x - logGamma a )
                        / evalContFractionB frac
@@ -356,37 +541,48 @@ invIncompleteGamma a p
 -- | Compute the natural logarithm of the beta function.
 --
 -- \[
--- B(a,b) = \int_0^1 t^{a-1}(1-t)^{1-b}\,dt = \frac{\Gamma(a)\Gamma(b)}{\Gamma(a+b)}
+-- B(a,b) = \int_0^1 t^{a-1}(1-t)^{b-1}\,dt = \frac{\Gamma(a)\Gamma(b)}{\Gamma(a+b)}
 -- \]
 logBeta
   :: Double                     -- ^ /a/ > 0
   -> Double                     -- ^ /b/ > 0
   -> Double
 logBeta a b
-    | p < 0     = m_NaN
-    | p == 0    = m_pos_inf
-    | p >= 10   = log q * (-0.5)
-                + m_ln_sqrt_2_pi
-                + logGammaCorrection p
-                + c
-                + (p - 0.5) * log ppq + q * log1p(-ppq)
-    | q >= 10   = logGamma p
-                + c
-                + p
-                - p * log pq
-                + (q - 0.5) * log1p(-ppq)
-    | otherwise = logGamma p + logGamma q - logGamma pq
-    where
-      p   = min a b
-      q   = max a b
-      ppq = p / pq
-      pq  = p + q
-      c   = logGammaCorrection q - logGammaCorrection pq
+  | p < 0     = m_NaN
+  | p == 0    = m_pos_inf
+  | p >= 10   = allStirling
+  | q >= 10   = twoStirling
+  -- This order of summands marginally improves precision
+  | otherwise = logGamma p + (logGamma q - logGamma pq)
+  where
+    p   = min a b
+    q   = max a b
+    ppq = p / pq
+    pq  = p + q
+    -- When both parameters are large than 10 we can use Stirling
+    -- approximation with correction. It's more precise than sum of
+    -- logarithms of gamma functions
+    allStirling
+      = log q * (-0.5)
+      + m_ln_sqrt_2_pi
+      + logGammaCorrection p
+      + (logGammaCorrection q - logGammaCorrection pq)
+      + (p - 0.5) * log ppq
+      + q * log1p(-ppq)
+    -- Otherwise only two of three gamma functions use Stirling
+    -- approximation
+    twoStirling
+      = logGamma p
+      + (logGammaCorrection q - logGammaCorrection pq)
+      + p
+      - p * log pq
+      + (q - 0.5) * log1p(-ppq)
+
 
 -- | Regularized incomplete beta function.
 --
 -- \[
--- I(x;a,b) = \frac{1}{B(a,b)} \int_0^x t^{a-1}(1-t)^{1-b}\,dt
+-- I(x;a,b) = \frac{1}{B(a,b)} \int_0^x t^{a-1}(1-t)^{b-1}\,dt
 -- \]
 --
 -- Uses algorithm AS63 by Majumder and Bhattachrjee and quadrature
@@ -722,65 +918,6 @@ sinc x
 -- Logarithm
 ----------------------------------------------------------------
 
--- GHC.Float provides log1p and expm1 since 4.9.0
-#if !MIN_VERSION_base(4,9,0)
--- | Compute the natural logarithm of 1 + @x@.  This is accurate even
--- for values of @x@ near zero, where use of @log(1+x)@ would lose
--- precision.
-log1p :: Double -> Double
-log1p x
-    | x == 0               = 0
-    | x == -1              = m_neg_inf
-    | x < -1               = m_NaN
-    | x' < m_epsilon * 0.5 = x
-    | (x >= 0 && x < 1e-8) || (x >= -1e-9 && x < 0)
-                           = x * (1 - x * 0.5)
-    | x' < 0.375           = x * (1 - x * chebyshevBroucke (x / 0.375) coeffs)
-    | otherwise            = log (1 + x)
-  where
-    x' = abs x
-    coeffs = U.fromList [
-               0.10378693562743769800686267719098e+1,
-              -0.13364301504908918098766041553133e+0,
-               0.19408249135520563357926199374750e-1,
-              -0.30107551127535777690376537776592e-2,
-               0.48694614797154850090456366509137e-3,
-              -0.81054881893175356066809943008622e-4,
-               0.13778847799559524782938251496059e-4,
-              -0.23802210894358970251369992914935e-5,
-               0.41640416213865183476391859901989e-6,
-              -0.73595828378075994984266837031998e-7,
-               0.13117611876241674949152294345011e-7,
-              -0.23546709317742425136696092330175e-8,
-               0.42522773276034997775638052962567e-9,
-              -0.77190894134840796826108107493300e-10,
-               0.14075746481359069909215356472191e-10,
-              -0.25769072058024680627537078627584e-11,
-               0.47342406666294421849154395005938e-12,
-              -0.87249012674742641745301263292675e-13,
-               0.16124614902740551465739833119115e-13,
-              -0.29875652015665773006710792416815e-14,
-               0.55480701209082887983041321697279e-15,
-              -0.10324619158271569595141333961932e-15
-             ]
-
--- | Compute @exp x - 1@ without loss of accuracy for x near zero.
-expm1 :: Double -> Double
-#ifdef USE_SYSTEM_EXPM1
-expm1 = c_expm1
-
-foreign import ccall "expm1" c_expm1 :: Double -> Double
-#else
--- NOTE: this is simplest implementation and not terribly efficient.
-expm1 x
-  | x < (-37.42994775023705) = -1
-  | x > m_max_log            = m_pos_inf
-  | abs x > 0.5              = exp x - 1
-  | otherwise                = sumSeries $ liftA2 (*) (scanSequence (*) x (pure x))
-                                                      (1 / scanSequence (*) 1 (enumSequenceFrom 2))
-#endif
-#endif
-
 -- | Compute log(1+x)-x:
 log1pmx :: Double -> Double
 log1pmx x
@@ -813,32 +950,33 @@ log2 v0
 -- Factorial
 ----------------------------------------------------------------
 
--- | Compute the factorial function /n/!.  Returns +∞ if the
--- input is above 170 (above which the result cannot be represented by
--- a 64-bit 'Double').
+-- | Compute the factorial function /n/!.  Returns +∞ if the input is
+--   above 170 (above which the result cannot be represented by a
+--   64-bit 'Double').
 factorial :: Int -> Double
 factorial n
-    | n < 0     = error "Numeric.SpecFunctions.factorial: negative input"
-    | n <= 1    = 1
-    | n <= 170  = U.product $ U.map fromIntegral $ U.enumFromTo 2 n
-    | otherwise = m_pos_inf
+  | n < 0     = error "Numeric.SpecFunctions.factorial: negative input"
+  | n > 170   = m_pos_inf
+  | otherwise = U.unsafeIndex factorialTable n
 
 -- | Compute the natural logarithm of the factorial function.  Gives
--- 16 decimal digits of precision.
+--   16 decimal digits of precision.
 logFactorial :: Integral a => a -> Double
 logFactorial n
-    | n <  0    = error "Numeric.SpecFunctions.logFactorial: negative input"
-    | n <= 14   = log $ factorial $ fromIntegral n
-    -- N.B. Γ(n+1) = n!
-    --
-    -- We use here asymptotic series for gamma function. See
-    -- http://mathworld.wolfram.com/StirlingsSeries.html
-    | otherwise = (x - 0.5) * log x - x
-                + m_ln_sqrt_2_pi
-                + evaluateOddPolynomialL (1/x) [1/12, -1/360, 1/1260, -1/1680]
-    where x = fromIntegral n + 1
+  | n <  0    = error "Numeric.SpecFunctions.logFactorial: negative input"
+  -- For smaller inputs we just look up table
+  | n <= 170  = log $ U.unsafeIndex factorialTable (fromIntegral n)
+  -- Otherwise we use asymptotic Stirling's series. Number of terms
+  -- necessary depends on the argument.
+  | n < 1500  = stirling + rx * ((1/12) - (1/360)*rx*rx)
+  | otherwise = stirling + (1/12)*rx
+  where
+    stirling = (x - 0.5) * log x - x + m_ln_sqrt_2_pi
+    x        = fromIntegral n + 1
+    rx       = 1 / x
 {-# SPECIALIZE logFactorial :: Int -> Double #-}
 
+
 -- | Calculate the error term of the Stirling approximation.  This is
 -- only defined for non-negative values.
 --
@@ -1008,3 +1146,179 @@ trigamma1 = 1.6449340668482264365 -- pi**2 / 6
 
 modErr :: String -> a
 modErr msg = error $ "Numeric.SpecFunctions." ++ msg
+
+factorialTable :: U.Vector Double
+{-# NOINLINE factorialTable #-}
+factorialTable = U.fromListN 171
+  [ 1.0
+  , 1.0
+  , 2.0
+  , 6.0
+  , 24.0
+  , 120.0
+  , 720.0
+  , 5040.0
+  , 40320.0
+  , 362880.0
+  , 3628800.0
+  , 3.99168e7
+  , 4.790016e8
+  , 6.2270208e9
+  , 8.71782912e10
+  , 1.307674368e12
+  , 2.0922789888e13
+  , 3.55687428096e14
+  , 6.402373705728e15
+  , 1.21645100408832e17
+  , 2.43290200817664e18
+  , 5.109094217170944e19
+  , 1.1240007277776077e21
+  , 2.5852016738884974e22
+  , 6.204484017332394e23
+  , 1.5511210043330984e25
+  , 4.032914611266056e26
+  , 1.0888869450418352e28
+  , 3.0488834461171384e29
+  , 8.841761993739702e30
+  , 2.6525285981219103e32
+  , 8.222838654177922e33
+  , 2.631308369336935e35
+  , 8.683317618811886e36
+  , 2.9523279903960412e38
+  , 1.0333147966386144e40
+  , 3.719933267899012e41
+  , 1.3763753091226343e43
+  , 5.23022617466601e44
+  , 2.0397882081197442e46
+  , 8.159152832478977e47
+  , 3.3452526613163803e49
+  , 1.4050061177528798e51
+  , 6.041526306337383e52
+  , 2.6582715747884485e54
+  , 1.1962222086548019e56
+  , 5.5026221598120885e57
+  , 2.5862324151116818e59
+  , 1.2413915592536073e61
+  , 6.082818640342675e62
+  , 3.0414093201713376e64
+  , 1.5511187532873822e66
+  , 8.065817517094388e67
+  , 4.2748832840600255e69
+  , 2.308436973392414e71
+  , 1.2696403353658275e73
+  , 7.109985878048634e74
+  , 4.0526919504877214e76
+  , 2.3505613312828785e78
+  , 1.386831185456898e80
+  , 8.32098711274139e81
+  , 5.075802138772247e83
+  , 3.146997326038793e85
+  , 1.9826083154044399e87
+  , 1.2688693218588415e89
+  , 8.24765059208247e90
+  , 5.44344939077443e92
+  , 3.647111091818868e94
+  , 2.4800355424368305e96
+  , 1.711224524281413e98
+  , 1.197857166996989e100
+  , 8.504785885678623e101
+  , 6.1234458376886085e103
+  , 4.470115461512684e105
+  , 3.307885441519386e107
+  , 2.4809140811395396e109
+  , 1.88549470166605e111
+  , 1.4518309202828586e113
+  , 1.1324281178206297e115
+  , 8.946182130782974e116
+  , 7.15694570462638e118
+  , 5.797126020747368e120
+  , 4.753643337012841e122
+  , 3.9455239697206583e124
+  , 3.314240134565353e126
+  , 2.81710411438055e128
+  , 2.422709538367273e130
+  , 2.1077572983795275e132
+  , 1.8548264225739844e134
+  , 1.650795516090846e136
+  , 1.4857159644817613e138
+  , 1.352001527678403e140
+  , 1.2438414054641305e142
+  , 1.1567725070816416e144
+  , 1.087366156656743e146
+  , 1.0329978488239058e148
+  , 9.916779348709496e149
+  , 9.619275968248211e151
+  , 9.426890448883246e153
+  , 9.332621544394413e155
+  , 9.332621544394415e157
+  , 9.425947759838358e159
+  , 9.614466715035125e161
+  , 9.902900716486179e163
+  , 1.0299016745145626e166
+  , 1.0813967582402908e168
+  , 1.1462805637347082e170
+  , 1.2265202031961378e172
+  , 1.3246418194518288e174
+  , 1.4438595832024934e176
+  , 1.5882455415227428e178
+  , 1.7629525510902446e180
+  , 1.974506857221074e182
+  , 2.2311927486598134e184
+  , 2.543559733472187e186
+  , 2.9250936934930154e188
+  , 3.393108684451898e190
+  , 3.9699371608087206e192
+  , 4.68452584975429e194
+  , 5.574585761207606e196
+  , 6.689502913449126e198
+  , 8.094298525273443e200
+  , 9.875044200833601e202
+  , 1.214630436702533e205
+  , 1.5061417415111406e207
+  , 1.8826771768889257e209
+  , 2.372173242880047e211
+  , 3.0126600184576594e213
+  , 3.856204823625804e215
+  , 4.974504222477286e217
+  , 6.466855489220473e219
+  , 8.471580690878819e221
+  , 1.1182486511960041e224
+  , 1.4872707060906857e226
+  , 1.9929427461615188e228
+  , 2.6904727073180504e230
+  , 3.6590428819525483e232
+  , 5.012888748274991e234
+  , 6.917786472619488e236
+  , 9.615723196941088e238
+  , 1.3462012475717523e241
+  , 1.898143759076171e243
+  , 2.6953641378881624e245
+  , 3.8543707171800725e247
+  , 5.5502938327393044e249
+  , 8.047926057471992e251
+  , 1.1749972043909107e254
+  , 1.7272458904546386e256
+  , 2.5563239178728654e258
+  , 3.808922637630569e260
+  , 5.713383956445854e262
+  , 8.62720977423324e264
+  , 1.3113358856834524e267
+  , 2.0063439050956823e269
+  , 3.0897696138473508e271
+  , 4.789142901463393e273
+  , 7.471062926282894e275
+  , 1.1729568794264143e278
+  , 1.8532718694937346e280
+  , 2.946702272495038e282
+  , 4.714723635992061e284
+  , 7.590705053947218e286
+  , 1.2296942187394494e289
+  , 2.0044015765453023e291
+  , 3.287218585534296e293
+  , 5.423910666131589e295
+  , 9.003691705778436e297
+  , 1.5036165148649988e300
+  , 2.526075744973198e302
+  , 4.269068009004705e304
+  , 7.257415615307998e306
+  ]
